
#include <cassert>  /* assert */
#include "client.h"
        
Client::Client(std::string name, unsigned int identifier,
           CurrentAccount         *a0,
           UnblockedSavingAccount *a1):
    m_name(name), m_identifier(identifier),
    m_current_account(a0), m_saving_account(a1) {
}

unsigned int Client::identifier() const {
    return m_identifier;
}

void Client::credit_current_account(double amount) {
    assert(m_current_account != nullptr);
    m_current_account->credit(amount);
}

void Client::credit_saving_account(double amount) {
    assert(m_saving_account != nullptr);
    m_saving_account->credit(amount);
}

double Client::debit_current_account(double amount) {
    assert(m_current_account != nullptr);
    return m_current_account->debit(amount);
}

double Client::debit_saving_account(double amount) {
    assert(m_saving_account != nullptr);
    return m_saving_account->debit(amount);
}

std::ostream &operator<<(std::ostream &os, const Client& client) {
    os << " [Client]" << std::endl;
    os << "  name:       " << client.m_name       << std::endl;
    os << "  identifier: " << client.m_identifier << std::endl;
    os << "  ";
    if (client.m_current_account) {
        client.m_current_account->print(os);
    }
    else {
        os << "-- no current account opened";
    }
    os << std::endl << "  ";
    if (client.m_saving_account) {
        client.m_saving_account->print(os);
    }
    else {
        os << "-- no saving account opened";
    }
    os << std::endl;
    return os;
}
